% list of all neighboring pairs of 2D array elements that allow valid moves 
% when generated with simple row/column electrodes

% KFB 03/23/03
function l = array2lines_neighbors(a)

a1 = a(:,1);  a2 = a(:,2);
a1diag = array_neighbors(a1,1);  a2diag = array_neighbors(a2,1);
% generate all valid moves for 2 droplets
l2 = array2_neighbors(a);
% eliminate all moves that cause conflicts with the row/column electrodes
s = ones(size(l2));
for i=1:length(l2)
    n = l2{i};
    n1 = n(:,1);  n2 = n(:,2); % these are the new coordinates for the 2 droplets
    % This means that:
    % - lines with x-coordinate n1(1) and n2(1) are activated (unless n1 == a1)
    % - lines with y-coordinate n1(2) and n2(2) are activated (unless n2 == a2).
    % This may cause 2 other activated coordinates:
    bad1 = [n1(1);n2(2)];  bad2 = [n2(1);n1(2)];
    % make sure these coordinates do not interfere
    if ~array_equal(n1,a1) & ~array_equal(n2,a2), % both droplets move
        % Note that this cannot be achieved by activating only one row and one column
        if ~array_equal(bad1,n1) & ~array_equal(bad1,n2),
            if any(cell_ismember(a1diag,bad1,@array_equal)), s(i) = 0; continue; end
            if any(cell_ismember(a2diag,bad1,@array_equal)), s(i) = 0; continue; end
        end
        if ~array_equal(bad2,n1) & ~array_equal(bad2,n2),
            if any(cell_ismember(a1diag,bad2,@array_equal)), s(i) = 0; continue; end
            if any(cell_ismember(a2diag,bad2,@array_equal)), s(i) = 0; continue; end
        end
    end
end
l = l2(find(s));