% A* search algorithm
% find the shortest path between start and goal using a prioritized breadth first search
% neighbor is a function that lists all nodes reachable in one transition
% estimate is a function that generates an admissible scalar that (under)estimates the distance between two nodes
% equal is a function that compares two nodes

% KFB 01/26/03, 03/14/03, 02/29/04, 04/13/04
function found = astar(start,goal,neighbor,estimate,equal)

global QUEUE;
dqueue; % create empty priority queue

global NODELIST;
nodelist; % create empty list of all visited nodes

global TIME; % time step (needed for algorithms with time-dependent obstacles)

current.node = start;  current.index = 1;  current.previous = 1;  current.value = 0;    % make the start node the current node
nodelist_add(current); 
dqueue_push(current,feval(estimate,start,goal)+1);
% here the distance estimate is assumed to be an integer value, which is used as the priority index
% since the distance can be 0, we have to add 1 to obtain a valid index value

found = []; % path from start to goal
while ~dqueue_empty,
    s = dqueue_pop; % get closest node
    TIME = s.value;
    if feval(equal,s.node,goal),    % goal reached!
        found = [{s.node}];
        i = s.index;
        while i~=1  % reconstruct path from start to goal
            i = s.previous;  p = nodelist_get(i);  s = p;
            found = [{s.node} found];
        end
        break;
    else    % goal not yet reached
        n = feval(neighbor,s.node); % generate list of neighbor nodes
        for i=1:length(n)
            next.node = n{i};  next.index = nodelist_length+1;  next.previous = s.index;  next.value = s.value+1;
            % check whether node has already been visited (this test may not be necessary - need to check!)
            if ~nodelist_in(next,@array_equal),
                nodelist_add(next);
                dqueue_push(next,next.value+feval(estimate,next.node,goal)+1);
            end
        end
    end
end