% A* search algorithm for multiple moving objects
% find the shortest path between start and goal using a prioritized breadth first search
% neighbor is a function that lists all nodes reachable in one transition
% estimate is a function that generates an admissible scalar that (under)estimates the distance between two nodes
% equal is a function that compares two nodes
%
% paths are found iteratively according to the list of (start,goal) pairs
% such that earlier paths become time-dependent constraints for later paths

% KNOWN BUGS:
% (1) if the path of a lower priority object lp is shorter than a path of a higher priority object hp, 
% then the algorithm currently does not check whether lp at the goal cell interferes with the remainder of the path of hp.
% (2) the NIL moves (object is just waiting) don't seem to work properly; some (or all?) solutions involving NIL are overlooked

% KFB 02/29/04
function found = astar(start,goal,neighbor,estimate,equal)

global PATH;
PATH = [];  % keep track of solution paths already found

pmax = size(start,2);
for p=1:pmax
    % find shortest path between start and goal for priority p
    path = astar(start(:,p),goal(:,p),neighbor,estimate,equal);
    % add new path to list of time-dependent obstacles
    if isempty(path)
        PATH = [];  break;
    else
        path_merge(path);
    end
end

found = PATH;